@@ -33,9 +33,7 @@ gmm = function(y,gen,dta=NULL,it=75,bi=25,th=1,model="BRR",...){
     KERN = TRUE
     if(model=="GBLUP"){
       idnames = rownames(gen)
-      gen = apply(gen,2,function(x) x-mean(x))
-      gen = tcrossprod(gen)
-      gen = gen/mean(diag(gen))
+      gen = GRM(gen)
       cat("Eigendecomposing GRM for GBLUP \n")
       gen = eigen(gen,TRUE)
       V = gen$values
@@ -45,8 +43,7 @@ gmm = function(y,gen,dta=NULL,it=75,bi=25,th=1,model="BRR",...){
     }
     if(model=="RKHS"){
       idnames = rownames(gen)
-      gen = as.matrix(dist(gen)^2)
-      gen = exp(-gen/median(gen))
+      gen = GAU(gen)
       cat("Eigendecomposing Gaussian Kernel for RKHS \n")
       gen = eigen(gen,TRUE)
       V = gen$values
@@ -330,13 +327,13 @@ gmm = function(y,gen,dta=NULL,it=75,bi=25,th=1,model="BRR",...){
         Va = (sum(g^2) + S_prior)/rchisq(1, df_prior + p)
         Vm = rep(Va,p)
         Ve = (crossprod(e)+Se_prior)/rchisq(1,n+df_prior)
-        L = Ve/Vm
+        L = c(Ve/Vm)
       }
       if(model=="BayesA"){
         Vm = (S_conj + g^2)/rchisq(p, df_prior + 1)
         S_conj = rgamma(1, p * df_prior/2 + shape_prior,sum(1/Vb)/2 + rate_prior)  
         Ve = (crossprod(e)+Se_prior)/rchisq(1,n+df_prior)
-        L = Ve/Vm
+        L = c(Ve/Vm)
       }
       if(KERN){
         Va = (sum(g^2/V) + Sk_prior)/rchisq(1, df_prior + p)
